@@ -1,6 +1,5 @@
-"""This view provides the main git-cola user interface.
-"""
-from __future__ import division, absolute_import, unicode_literals
+"""Main UI for authoring commits and other Git Cola interactions"""
+from __future__ import absolute_import, division, print_function, unicode_literals
 import os
 from functools import partial
 
@@ -64,7 +63,7 @@ class MainView(standard.MainWindow):
     updated = Signal()
 
     def __init__(self, context, parent=None):
-        # pylint: disable=too-many-statements
+        # pylint: disable=too-many-statements,too-many-locals
         standard.MainWindow.__init__(self, parent)
         self.setAttribute(Qt.WA_DeleteOnClose)
 
@@ -86,16 +85,19 @@ class MainView(standard.MainWindow):
             browser = browse.worktree_browser(
                 context, parent=self, show=False, update=False
             )
-            self.browserdock = create_dock(N_('Browser'), self, widget=browser)
+            self.browserdock = create_dock(
+                'Browser', N_('Browser'), self, widget=browser
+            )
 
         # "Actions" widget
         self.actionsdock = create_dock(
-            N_('Actions'), self, widget=action.ActionButtons(context, self)
+            'Actions', N_('Actions'), self, widget=action.ActionButtons(context, self)
         )
         qtutils.hide_dock(self.actionsdock)
 
         # "Repository Status" widget
         self.statusdock = create_dock(
+            'Status',
             N_('Status'),
             self,
             fn=lambda dock: status.StatusWidget(context, dock.titleBarWidget(), dock),
@@ -104,13 +106,19 @@ class MainView(standard.MainWindow):
 
         # "Switch Repository" widgets
         self.bookmarksdock = create_dock(
-            N_('Favorites'), self, fn=lambda dock: bookmarks.bookmark(context, dock)
+            'Favorites',
+            N_('Favorites'),
+            self,
+            fn=lambda dock: bookmarks.bookmark(context, dock)
         )
         bookmarkswidget = self.bookmarksdock.widget()
         qtutils.hide_dock(self.bookmarksdock)
 
         self.recentdock = create_dock(
-            N_('Recent'), self, fn=lambda dock: bookmarks.recent(context, dock)
+            'Recent',
+            N_('Recent'),
+            self,
+            fn=lambda dock: bookmarks.recent(context, dock)
         )
         recentwidget = self.recentdock.widget()
         qtutils.hide_dock(self.recentdock)
@@ -118,7 +126,10 @@ class MainView(standard.MainWindow):
 
         # "Branch" widgets
         self.branchdock = create_dock(
-            N_('Branches'), self, fn=partial(branch.BranchesWidget, context)
+            'Branches',
+            N_('Branches'),
+            self,
+            fn=partial(branch.BranchesWidget, context)
         )
         self.branchwidget = self.branchdock.widget()
         titlebar = self.branchdock.titleBarWidget()
@@ -127,7 +138,10 @@ class MainView(standard.MainWindow):
 
         # "Submodule" widgets
         self.submodulesdock = create_dock(
-            N_('Submodules'), self, fn=partial(submodules.SubmodulesWidget, context)
+            'Submodules',
+            N_('Submodules'),
+            self,
+            fn=partial(submodules.SubmodulesWidget, context)
         )
         self.submoduleswidget = self.submodulesdock.widget()
 
@@ -145,18 +159,20 @@ class MainView(standard.MainWindow):
 
         editor = commitmsg.CommitMessageEditor(context, self)
         self.commiteditor = editor
-        self.commitdock = create_dock(N_('Commit'), self, widget=editor)
+        self.commitdock = create_dock('Commit', N_('Commit'), self, widget=editor)
         titlebar = self.commitdock.titleBarWidget()
         titlebar.add_corner_widget(self.position_label)
 
         # "Console" widget
         self.logwidget = log.LogWidget(context)
-        self.logdock = create_dock(N_('Console'), self, widget=self.logwidget)
+        self.logdock = create_dock(
+            'Console', N_('Console'), self, widget=self.logwidget
+        )
         qtutils.hide_dock(self.logdock)
 
         # "Diff Viewer" widget
         self.diffdock = create_dock(
-            N_('Diff'), self, fn=lambda dock: diff.Viewer(context, parent=dock)
+            'Diff', N_('Diff'), self, fn=lambda dock: diff.Viewer(context, parent=dock)
         )
         self.diffviewer = self.diffdock.widget()
         self.diffviewer.set_diff_type(self.model.diff_type)
@@ -185,8 +201,7 @@ class MainView(standard.MainWindow):
         self.unstage_all_action.setIcon(icons.remove())
 
         self.undo_commit_action = add_action(
-            self, N_('Undo Last Commit'),
-            cmds.run(cmds.UndoLastCommit, context)
+            self, N_('Undo Last Commit'), cmds.run(cmds.UndoLastCommit, context)
         )
         self.undo_commit_action.setIcon(icons.style_dialog_discard())
 
@@ -219,7 +234,7 @@ class MainView(standard.MainWindow):
         self.apply_patches_action = add_action(
             self, N_('Apply Patches...'), partial(patch.apply_patches, context)
         )
-        self.apply_patches_action.setIcon(icons.style_dialog_apply())
+        self.apply_patches_action.setIcon(icons.diff())
 
         self.export_patches_action = add_action(
             self,
@@ -245,6 +260,7 @@ class MainView(standard.MainWindow):
         self.preferences_action = add_action(
             self, N_('Preferences'), prefs_fn, QtGui.QKeySequence.Preferences
         )
+        self.preferences_action.setIcon(icons.configure())
 
         self.edit_remotes_action = add_action(
             self, N_('Edit Remotes...'), partial(editremotes.editor, context)
@@ -266,7 +282,7 @@ class MainView(standard.MainWindow):
             hotkeys.FINDER,
             hotkeys.FINDER_SECONDARY,
         )
-        self.find_files_action.setIcon(icons.zoom_in())
+        self.find_files_action.setIcon(icons.search())
 
         self.browse_recently_modified_action = add_action(
             self,
@@ -274,7 +290,7 @@ class MainView(standard.MainWindow):
             partial(recent.browse_recent_files, context),
             hotkeys.EDIT_SECONDARY,
         )
-        self.browse_recently_modified_action.setIcon(icons.edit())
+        self.browse_recently_modified_action.setIcon(icons.directory())
 
         self.cherry_pick_action = add_action(
             self,
@@ -282,13 +298,12 @@ class MainView(standard.MainWindow):
             partial(guicmds.cherry_pick, context),
             hotkeys.CHERRY_PICK,
         )
-        self.cherry_pick_action.setIcon(icons.style_dialog_apply())
+        self.cherry_pick_action.setIcon(icons.cherry_pick())
 
         self.load_commitmsg_action = add_action(
-            self,
-            N_('Load Commit Message...'),
-            partial(guicmds.load_commitmsg, context)
+            self, N_('Load Commit Message...'), partial(guicmds.load_commitmsg, context)
         )
+        self.load_commitmsg_action.setIcon(icons.file_text())
 
         self.prepare_commitmsg_hook_action = add_action(
             self,
@@ -298,9 +313,7 @@ class MainView(standard.MainWindow):
         )
 
         self.save_tarball_action = add_action(
-            self,
-            N_('Save As Tarball/Zip...'),
-            partial(archive.save_archive, context)
+            self, N_('Save As Tarball/Zip...'), partial(archive.save_archive, context)
         )
         self.save_tarball_action.setIcon(icons.file_zip())
 
@@ -368,43 +381,45 @@ class MainView(standard.MainWindow):
         self.stash_action.setIcon(icons.commit())
 
         self.reset_soft_action = add_action(
-            self, N_('Reset Branch (Soft)'),
-            partial(guicmds.reset_soft, context)
+            self, N_('Reset Branch (Soft)'), partial(guicmds.reset_soft, context)
         )
         self.reset_soft_action.setIcon(icons.style_dialog_reset())
         self.reset_soft_action.setToolTip(cmds.ResetSoft.tooltip('<commit>'))
 
         self.reset_mixed_action = add_action(
-            self, N_('Reset Branch and Stage (Mixed)'),
-            partial(guicmds.reset_mixed, context)
+            self,
+            N_('Reset Branch and Stage (Mixed)'),
+            partial(guicmds.reset_mixed, context),
         )
         self.reset_mixed_action.setIcon(icons.style_dialog_reset())
         self.reset_mixed_action.setToolTip(cmds.ResetMixed.tooltip('<commit>'))
 
         self.reset_keep_action = add_action(
-            self, N_('Restore Worktree and Reset All (Keep Unstaged Changes)'),
-            partial(guicmds.reset_keep, context)
+            self,
+            N_('Restore Worktree and Reset All (Keep Unstaged Changes)'),
+            partial(guicmds.reset_keep, context),
         )
         self.reset_keep_action.setIcon(icons.style_dialog_reset())
         self.reset_keep_action.setToolTip(cmds.ResetKeep.tooltip('<commit>'))
 
         self.reset_merge_action = add_action(
-            self, N_('Restore Worktree and Reset All (Merge)'),
-            partial(guicmds.reset_merge, context)
+            self,
+            N_('Restore Worktree and Reset All (Merge)'),
+            partial(guicmds.reset_merge, context),
         )
         self.reset_merge_action.setIcon(icons.style_dialog_reset())
         self.reset_merge_action.setToolTip(cmds.ResetMerge.tooltip('<commit>'))
 
         self.reset_hard_action = add_action(
-            self, N_('Restore Worktree and Reset All (Hard)'),
-            partial(guicmds.reset_hard, context)
+            self,
+            N_('Restore Worktree and Reset All (Hard)'),
+            partial(guicmds.reset_hard, context),
         )
         self.reset_hard_action.setIcon(icons.style_dialog_reset())
         self.reset_hard_action.setToolTip(cmds.ResetHard.tooltip('<commit>'))
 
         self.restore_worktree_action = add_action(
-            self, N_('Restore Worktree'),
-            partial(guicmds.restore_worktree, context)
+            self, N_('Restore Worktree'), partial(guicmds.restore_worktree, context)
         )
         self.restore_worktree_action.setIcon(icons.edit())
         self.restore_worktree_action.setToolTip(
@@ -449,14 +464,20 @@ class MainView(standard.MainWindow):
             N_('Browse Current Branch...'),
             partial(guicmds.browse_current, context),
         )
+        self.browse_branch_action.setIcon(icons.directory())
+
         self.browse_other_branch_action = add_action(
             self, N_('Browse Other Branch...'), partial(guicmds.browse_other, context)
         )
+        self.browse_other_branch_action.setIcon(icons.directory())
+
         self.load_commitmsg_template_action = add_action(
             self,
             N_('Get Commit Message Template'),
             cmds.run(cmds.LoadCommitMessageFromTemplate, context),
         )
+        self.load_commitmsg_template_action.setIcon(icons.style_dialog_apply())
+
         self.help_about_action = add_action(
             self, N_('About'), partial(about.about_dialog, context)
         )
@@ -509,6 +530,8 @@ class MainView(standard.MainWindow):
             partial(guicmds.checkout_branch, context),
             hotkeys.CHECKOUT,
         )
+        self.checkout_branch_action.setIcon(icons.branch())
+
         self.branch_review_action = add_action(
             self, N_('Review...'), partial(guicmds.review_branch, context)
         )
@@ -633,23 +656,30 @@ class MainView(standard.MainWindow):
             bookmarkswidget.tree,
             recentwidget.tree,
         )
+        select_widgets = copy_widgets + (self.statuswidget.tree,)
         edit_proxy.override('copy', copy_widgets)
-        edit_proxy.override('selectAll', copy_widgets)
+        edit_proxy.override('selectAll', select_widgets)
 
         edit_menu = self.edit_menu = add_menu(N_('&Edit'), self.menubar)
-        add_action(edit_menu, N_('Undo'), edit_proxy.undo, hotkeys.UNDO)
-        add_action(edit_menu, N_('Redo'), edit_proxy.redo, hotkeys.REDO)
+        undo = add_action(edit_menu, N_('Undo'), edit_proxy.undo, hotkeys.UNDO)
+        undo.setIcon(icons.undo())
+        redo = add_action(edit_menu, N_('Redo'), edit_proxy.redo, hotkeys.REDO)
+        redo.setIcon(icons.redo())
         edit_menu.addSeparator()
-        add_action(edit_menu, N_('Cut'), edit_proxy.cut, hotkeys.CUT)
-        add_action(edit_menu, N_('Copy'), edit_proxy.copy, hotkeys.COPY)
-        add_action(edit_menu, N_('Paste'), edit_proxy.paste, hotkeys.PASTE)
-        add_action(edit_menu, N_('Delete'), edit_proxy.delete, hotkeys.DELETE)
+        cut = add_action(edit_menu, N_('Cut'), edit_proxy.cut, hotkeys.CUT)
+        cut.setIcon(icons.cut())
+        copy = add_action(edit_menu, N_('Copy'), edit_proxy.copy, hotkeys.COPY)
+        copy.setIcon(icons.copy())
+        paste = add_action(edit_menu, N_('Paste'), edit_proxy.paste, hotkeys.PASTE)
+        paste.setIcon(icons.paste())
+        delete = add_action(edit_menu, N_('Delete'), edit_proxy.delete, hotkeys.DELETE)
+        delete.setIcon(icons.delete())
         edit_menu.addSeparator()
-        add_action(
+        select_all = add_action(
             edit_menu, N_('Select All'), edit_proxy.selectAll, hotkeys.SELECT_ALL
         )
+        select_all.setIcon(icons.select_all())
         edit_menu.addSeparator()
-
         commitmsg.add_menu_actions(edit_menu, self.commiteditor.menu_actions)
 
         # Actions menu
@@ -677,6 +707,8 @@ class MainView(standard.MainWindow):
         self.commit_menu.addAction(self.commit_amend_action)
         self.commit_menu.addAction(self.undo_commit_action)
         self.commit_menu.addSeparator()
+        self.commit_menu.addAction(self.statuswidget.tree.process_selection_action)
+        self.commit_menu.addAction(self.statuswidget.tree.stage_or_unstage_all_action)
         self.commit_menu.addAction(self.stage_modified_action)
         self.commit_menu.addAction(self.stage_untracked_action)
         self.commit_menu.addSeparator()
@@ -799,7 +831,12 @@ class MainView(standard.MainWindow):
         self.updated.connect(self.refresh, type=Qt.QueuedConnection)
 
         self.config_actions_changed.connect(
-            self._install_config_actions, type=Qt.QueuedConnection
+            lambda names_and_shortcuts: _install_config_actions(
+                context,
+                self.actions_menu,
+                names_and_shortcuts,
+            ),
+            type=Qt.QueuedConnection,
         )
         self.init_state(context.settings, self.set_initial_size)
 
@@ -866,12 +903,12 @@ class MainView(standard.MainWindow):
             menu_action.setParent(menu)
             menu.addAction(menu_action)
 
-        menu.addSeparator()
         context = self.context
         menu_action = menu.addAction(
-            N_('Add Toolbar'), partial(toolbar.add_toolbar, context, self)
+            N_('New Toolbar'), partial(toolbar.add_toolbar, context, self)
         )
         menu_action.setIcon(icons.add())
+        menu.addSeparator()
 
         dockwidgets = [
             self.logdock,
@@ -971,19 +1008,6 @@ class MainView(standard.MainWindow):
         actions = cfgactions.get_config_actions(self.context)
         self.config_actions_changed.emit(actions)
 
-    def _install_config_actions(self, names_and_shortcuts):
-        """Install .gitconfig-defined actions"""
-        if not names_and_shortcuts:
-            return
-        context = self.context
-        menu = self.actions_menu
-        menu.addSeparator()
-        for (name, shortcut) in names_and_shortcuts:
-            callback = cmds.run(cmds.RunConfigAction, context, name)
-            menu_action = menu.addAction(name, callback)
-            if shortcut:
-                menu_action.setShortcut(shortcut)
-
     def refresh(self):
         """Update the title with the current branch and directory name."""
         curbranch = self.model.currentbranch
@@ -1253,3 +1277,55 @@ def focus_dock(dockwidget):
         show_dock(dockwidget)
     else:
         dockwidget.toggleViewAction().trigger()
+
+
+def _install_config_actions(context, menu, names_and_shortcuts):
+    """Install .gitconfig-defined actions"""
+    if not names_and_shortcuts:
+        return
+    menu.addSeparator()
+    cache = {}
+    for (name, shortcut) in names_and_shortcuts:
+        sub_menu, action_name = build_menus(name, menu, cache)
+        callback = cmds.run(cmds.RunConfigAction, context, name)
+        menu_action = sub_menu.addAction(action_name, callback)
+        if shortcut:
+            menu_action.setShortcut(shortcut)
+
+
+def build_menus(name, menu, cache):
+    """Create a chain of QMenu entries parented under a root QMenu
+
+    A name of "a/b/c" create a menu chain of menu -> QMenu("a") -> QMenu("b")
+    and returns a tuple of (QMenu("b"), "c").
+
+    :param name: The full entry path, ex: "a/b/c" where "a/b" is the menu chain.
+    :param menu: The root menu under which to create the menu chain.
+    :param cache: A dict cache of previously created menus to avoid duplicates.
+
+    """
+    # NOTE: utils.split() and friends are used instead of os.path.split() because
+    # slash '/' is the only supported "<menu>/name" separator.  Use of os.path.split()
+    # would introduce differences in behavior across platforms.
+
+    # If the menu_path is empty then no parent menus need to be created.
+    # The action will be added to the root menu.
+    menu_path, text = utils.split(utils.normalize_slash(name))
+
+    if not menu_path:
+        return (menu, text)
+    # When menu_path contains ex: "a/b" we will create two menus: "a" and "b".
+    # The root menu is the parent of "a" and "a" is the parent of "b".
+    # The menu returned to the caller is "b".
+    #
+    # Loop over the individual menu basenames alongside the full subpath returned by
+    # pathset(). The subpath is a cache key for finding previously created menus.
+    menu_names = utils.splitpath(menu_path)  # ['a', 'b']
+    menu_pathset = utils.pathset(menu_path)  # ['a', 'a/b']
+    for menu_name, menu_id in zip(menu_names, menu_pathset):
+        try:
+            menu = cache[menu_id]
+        except KeyError:
+            menu = cache[menu_id] = menu.addMenu(menu_name)
+
+    return (menu, text)
