@@ -1,5 +1,5 @@
 """Provides High DPI support by wrapping Qt options"""
-from __future__ import absolute_import, division, unicode_literals
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 from qtpy import QtCore
 
@@ -26,8 +26,9 @@ def apply_choice(value):
     if value == Option.AUTO:
         # Do not override the configuration when either of these
         # two environment variables are defined.
-        if (not core.getenv('QT_AUTO_SCREEN_SCALE_FACTOR')
-                and not core.getenv('QT_SCALE_FACTOR')):
+        if not core.getenv('QT_AUTO_SCREEN_SCALE_FACTOR') and not core.getenv(
+            'QT_SCALE_FACTOR'
+        ):
             compat.setenv('QT_AUTO_SCREEN_SCALE_FACTOR', '1')
             compat.unsetenv('QT_SCALE_FACTOR')
     elif value in (Option.TIMES_1, Option.TIMES_1_5, Option.TIMES_2):
