@@ -1,4 +1,4 @@
-from __future__ import absolute_import
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 from qtpy.QtGui import QKeySequence
 from qtpy.QtCore import Qt
@@ -57,6 +57,7 @@ REFRESH = hotkey(Qt.CTRL + Qt.Key_R)
 REFRESH_SECONDARY = hotkey(Qt.Key_F5)
 REFRESH_HOTKEYS = (REFRESH, REFRESH_SECONDARY)
 STAGE_DIFF = hotkey(Qt.Key_S)
+STAGE_DIFF_ALT = hotkey(Qt.SHIFT + Qt.Key_S)
 STAGE_SELECTION = hotkey(Qt.CTRL + Qt.Key_S)
 STAGE_ALL = hotkey(Qt.CTRL + Qt.SHIFT + Qt.Key_S)
 STASH = hotkey(Qt.ALT + Qt.SHIFT + Qt.Key_S)
