@@ -1,5 +1,5 @@
 """A widget for searching git commits"""
-from __future__ import division, absolute_import, unicode_literals
+from __future__ import absolute_import, division, print_function, unicode_literals
 import time
 
 from qtpy import QtCore
@@ -76,7 +76,7 @@ class SearchWidget(standard.Dialog):
         )
 
         self.button_cherrypick = qtutils.create_button(
-            text=N_('Cherry Pick'), icon=icons.save()
+            text=N_('Cherry Pick'), icon=icons.cherry_pick()
         )
         self.button_close = qtutils.close_button()
 
